SystemOrganization addCategory: #'Container-Abstract'!
SystemOrganization addCategory: #'Container-Exceptions'!
SystemOrganization addCategory: #'Container-Iterators'!
SystemOrganization addCategory: #'Container-Lists'!
SystemOrganization addCategory: #'Container-Sets'!
SystemOrganization addCategory: #'Container-Maps'!
SystemOrganization addCategory: #'Container-Tests-Core'!

TestCase subclass: #CTCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Tests-Core'!

!CTCollectionTest class methodsFor: 'testing' stamp: 'lr 8/7/2011 19:59'!
isAbstract
	^ self name = #CTCollectionTest! !

!CTCollectionTest methodsFor: 'accessing' stamp: 'lr 11/5/2011 18:42'!
collectionClass
	^ CTCollection! !

!CTCollectionTest methodsFor: 'accessing' stamp: 'lr 8/7/2011 19:17'!
emptyCollection
	^ self collectionClass new! !

!CTCollectionTest methodsFor: 'accessing' stamp: 'lr 8/7/2011 19:18'!
oneElementCollection
	^ self collectionClass with: 1! !

!CTCollectionTest methodsFor: 'test-testing' stamp: 'lr 8/7/2011 19:19'!
testIsEmpty
	self assert: self emptyCollection isEmpty.
	self deny: self oneElementCollection isEmpty.
	self deny: self twoElementCollection isEmpty! !

!CTCollectionTest methodsFor: 'test-accessing' stamp: 'lr 8/7/2011 19:19'!
testSize
	self assert: self emptyCollection size = 0.
	self assert: self oneElementCollection size = 1.
	self assert: self twoElementCollection size = 2! !

!CTCollectionTest methodsFor: 'accessing' stamp: 'lr 8/7/2011 19:25'!
twoElementCollection
	^ self collectionClass with: 1 with: 2! !

CTCollectionTest subclass: #CTListTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Tests-Core'!

CTListTest subclass: #CTArrayListTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Tests-Core'!

!CTArrayListTest methodsFor: 'accessing' stamp: 'lr 8/7/2011 19:21'!
collectionClass
	^ CTArrayList! !

CTListTest subclass: #CTLinkedListTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Tests-Core'!

!CTLinkedListTest methodsFor: 'accessing' stamp: 'lr 8/7/2011 19:21'!
collectionClass
	^ CTLinkedList! !

!CTListTest class methodsFor: 'testing' stamp: 'lr 8/7/2011 19:59'!
isAbstract
	^ self name = #CTListTest! !

!CTListTest methodsFor: 'accessing' stamp: 'lr 11/5/2011 18:42'!
collectionClass
	^ CTList! !

!CTListTest methodsFor: 'test-accessing' stamp: 'lr 8/7/2011 20:08'!
testFirst
	self should: [ self emptyCollection first ] raise: CTNoSuchElementError.
	self assert: self oneElementCollection first = 1.
	self assert: self twoElementCollection first = 1! !

!CTListTest methodsFor: 'test-accessing' stamp: 'lr 8/7/2011 19:54'!
testLast
	self should: [ self emptyCollection last ] raise: CTNoSuchElementError.
	self assert: self oneElementCollection last = 1.
	self assert: self twoElementCollection last = 2! !

Object subclass: #CTCollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Abstract'!

!CTCollection class methodsFor: 'instance-creation' stamp: 'lr 8/7/2011 19:24'!
with: anObject1
	^ (self new: 1)
		add: anObject1;
		yourself! !

!CTCollection class methodsFor: 'instance-creation' stamp: 'lr 8/7/2011 19:24'!
with: anObject1 with: anObject2
	^ (self new: 2) 
		add: anObject1;
		add: anObject2;
		yourself! !

!CTCollection class methodsFor: 'instance-creation' stamp: 'lr 8/7/2011 19:24'!
with: anObject1 with: anObject2 with: anObject3
	^ (self new: 3) 
		add: anObject1;
		add: anObject2;
		add: anObject3;
		yourself! !

!CTCollection class methodsFor: 'instance-creation' stamp: 'lr 8/7/2011 19:24'!
with: anObject1 with: anObject2 with: anObject3 with: anObject4
	^ (self new: 4) 
		add: anObject1;
		add: anObject2;
		add: anObject3;
		add: anObject4;
		yourself! !

!CTCollection methodsFor: 'adding' stamp: 'lr 8/7/2011 11:03'!
add: anObject
	"Ensures that the receiver contains anObject."
	
	self subclassResponsibility! !

!CTCollection methodsFor: 'adding' stamp: 'lr 8/7/2011 11:03'!
addAll: aCollection
	"Ensures that the receiver contains all elements of aCollection."
	
	aCollection iterator do: [ :each | self add: each ]! !

!CTCollection methodsFor: 'private' stamp: 'lr 8/7/2011 11:07'!
elementNotFound: anObject
	^ CTElementNotFoundError signal! !

!CTCollection methodsFor: 'testing' stamp: 'lr 8/7/2011 11:05'!
includes: anObject
	"Returns true if the receiver contains anObject."
	
	self iterator do: [ :each |
		anObject = each
			ifTrue: [ ^ true ] ].
	^ false! !

!CTCollection methodsFor: 'private' stamp: 'lr 11/5/2011 18:43'!
indexOutOfBounds: anInteger
	^ CTIndexOutOfBoundsError signal! !

!CTCollection methodsFor: 'testing' stamp: 'lr 6/7/2011 19:15'!
isEmpty
	"Answer whether the receiver contains any elements."
	
	^ self size = 0! !

!CTCollection methodsFor: 'accessing' stamp: 'lr 8/7/2011 11:00'!
iterator
	"Returns a default iterator over the elements in this collection."

	self subclassResponsibility! !

!CTCollection methodsFor: 'private' stamp: 'lr 8/7/2011 19:55'!
noSuchElement
	^ CTNoSuchElementError signal! !

!CTCollection methodsFor: 'private' stamp: 'lr 8/7/2011 11:03'!
remove: anObject
	"Removes anObject from the receiver, throw an error if the anObject is not present."
	
	self remove: anObject ifAbsent: [ self elementNotFound: anObject ]! !

!CTCollection methodsFor: 'removing' stamp: 'lr 8/7/2011 11:04'!
remove: anObject ifAbsent: aBlock
	"Removes anObject from the receiver, evaluate aBlock if anObject is not present."
	
	self subclassResponsibility! !

!CTCollection methodsFor: 'accessing' stamp: 'lr 6/7/2011 19:15'!
size
	"Returns the number of elements in this collection."
	
	self subclassResponsibility! !

CTCollection subclass: #CTList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Abstract'!

CTList subclass: #CTArrayList
	instanceVariableNames: 'array firstIndex lastIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Lists'!

!CTArrayList class methodsFor: 'instance creation' stamp: 'lr 8/7/2011 19:10'!
new
	^ self new: 10! !

!CTArrayList class methodsFor: 'instance creation' stamp: 'lr 8/7/2011 19:10'!
new: anInteger
	^ self basicNew initialize: anInteger! !

!CTArrayList methodsFor: 'adding' stamp: 'lr 11/6/2011 17:12'!
add: anObject at: anInteger
	| target |
	anInteger = 1
		ifTrue: [ ^ self addFirst: anObject ].
	anInteger + 1 = self size
		ifTrue: [ ^ self addLast: anObject ].
	target := anInteger + firstIndex - 1.
	(target between: firstIndex and: lastIndex)
		ifFalse: [ ^ self indexOutOfBounds: anInteger ].
	array size // 2 < target
		ifTrue: [
			lastIndex = array size
				ifTrue: [ self growAtLast ].
			lastIndex to: target by: -1
				do: [ :index | array at: index + 1 put: (array at: index) ].
			lastIndex := lastIndex + 1 ]
		ifFalse: [ 
			firstIndex = 1 ifTrue: [
				self growAtFirst.
				target := anInteger + firstIndex - 1 ].
			firstIndex to: target - 1 by: 1
				do: [ :index | array at: index put: (array at: index + 1) ].
			firstIndex := firstIndex - 1 ].
	array at: target put: anObject.
	^ anObject
		! !

!CTArrayList methodsFor: 'adding' stamp: 'lr 8/7/2011 11:38'!
addFirst: anObject
	firstIndex = 1
		ifTrue: [ self growAtFirst ].
	firstIndex := firstIndex - 1.
	array at: firstIndex put: anObject! !

!CTArrayList methodsFor: 'adding' stamp: 'lr 8/7/2011 11:38'!
addLast: anObject
	lastIndex = array size
		ifTrue: [ self growAtLast ].
	lastIndex := lastIndex + 1.
	array at: lastIndex put: anObject! !

!CTArrayList methodsFor: 'accessing' stamp: 'lr 11/6/2011 16:48'!
at: anInteger ifAbsent: aBlock
	| index |
	index := anInteger + firstIndex - 1.
	(index between: firstIndex and: lastIndex)
		ifFalse: [ ^ aBlock value ].
	^ array at: index! !

!CTArrayList methodsFor: 'accessing' stamp: 'lr 11/6/2011 16:48'!
at: anInteger put: anObject
	| index |
	index := anInteger + firstIndex - 1.
	(index between: firstIndex and: lastIndex)
		ifFalse: [ ^  self indexOutOfBounds: anInteger ].
	^ array at: index put: anObject! !

!CTArrayList methodsFor: 'private' stamp: 'lr 8/7/2011 11:42'!
growAtFirst
	| newArray newFirstIndex newLastIndex |
	newArray := Array new: array size * 3 // 2 + 1.
	newFirstIndex := newArray size - array size + firstIndex.
	newLastIndex := newFirstIndex + lastIndex - firstIndex.
	newArray
		replaceFrom: newFirstIndex
		to: newLastIndex
		with: array
		startingAt: firstIndex.
	array := newArray.
	firstIndex := newFirstIndex.
	lastIndex := newLastIndex! !

!CTArrayList methodsFor: 'private' stamp: 'lr 8/7/2011 11:42'!
growAtLast
	| newArray |
	newArray := Array new: array size * 3 // 2 + 1.
	newArray
		replaceFrom: firstIndex
		to: lastIndex
		with: array
		startingAt: firstIndex.
	array := newArray! !

!CTArrayList methodsFor: 'initialization' stamp: 'lr 8/7/2011 19:09'!
initialize: anInteger
	self initialize.
	array := Array new: anInteger.
	firstIndex := 1.
	lastIndex := 0! !

!CTArrayList methodsFor: 'accessing' stamp: 'lr 11/6/2011 09:28'!
iterator
	"Returns a default iterator over the elements in this collection."

	^ CTArrayListIterator on: self! !

!CTArrayList methodsFor: 'removing' stamp: 'lr 11/6/2011 17:02'!
removeAt: anInteger ifAbsent: aBlock
	| target object |
	target := firstIndex + anInteger - 1.
	(target between: firstIndex and: lastIndex)
		ifFalse: [ ^ self indexOutOfBounds: anInteger ].
	object := array at: target.
	array size // 2 < target
		ifTrue: [ 
			target to: lastIndex - 1 by: 1
				do: [ :index | array at: index put: (array at: index + 1) ].
			array at: lastIndex put: nil.
			lastIndex := lastIndex - 1 ]
		ifFalse: [
			target to: firstIndex + 1 by: -1
				do: [ :index | array at: index - 1 put: (array at: index) ].
			array at: firstIndex put: nil.
			firstIndex := firstIndex + 1 ].
	^ object! !

!CTArrayList methodsFor: 'removing' stamp: 'lr 11/6/2011 16:54'!
removeFirst
	| element |
	self isEmpty
		ifFalse: [ ^ self noSuchElement ].
	element := array at: firstIndex.
	array at: firstIndex put: nil.
	firstIndex := firstIndex + 1.
	^ element! !

!CTArrayList methodsFor: 'removing' stamp: 'lr 11/6/2011 16:55'!
removeLast
	| element |
	self isEmpty
		ifFalse: [ ^ self noSuchElement ].
	element := array at: lastIndex.
	array at: lastIndex put: nil.
	lastIndex := lastIndex - 1.
	^ element! !

!CTArrayList methodsFor: 'accessing' stamp: 'lr 8/7/2011 16:16'!
size
	"Returns the number of elements in this collection."
	
	^ lastIndex - firstIndex + 1! !

CTList subclass: #CTLinkedList
	instanceVariableNames: 'size root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Lists'!

!CTLinkedList class methodsFor: 'instance creation' stamp: 'lr 8/7/2011 20:04'!
new
	^ self basicNew initialize! !

!CTLinkedList class methodsFor: 'instance creation' stamp: 'lr 8/7/2011 20:04'!
new: anInteger
	^ self new! !

!CTLinkedList methodsFor: 'adding' stamp: 'lr 11/6/2011 17:24'!
add: anObject at: anInteger
	| node |
	anInteger = 1
		ifTrue: [ ^ self addFirst: anObject ].
	anInteger = (self size + 1)
		ifTrue: [ ^ self addLast: anObject ].
	node := self nodeAt: anInteger.
	node isNil
		ifTrue: [ self indexOutOfBounds: anInteger ].
	self nodeAdd: (self nodeOn: anObject) before: node.
	^ anObject! !

!CTLinkedList methodsFor: 'adding' stamp: 'lr 11/6/2011 17:31'!
addFirst: anObject
	self nodeAdd: (self nodeOn: anObject) before: root after.
	^ anObject! !

!CTLinkedList methodsFor: 'adding' stamp: 'lr 11/6/2011 17:24'!
addLast: anObject
	self nodeAdd: (self nodeOn: anObject) before: root.
	^ anObject! !

!CTLinkedList methodsFor: 'accessing' stamp: 'lr 11/6/2011 17:28'!
at: anInteger ifAbsent: aBlock
	| node |
	node := self nodeAt: anInteger.
	^ node isNil
		ifTrue: [ aBlock value ]
		ifFalse: [ node object ]! !

!CTLinkedList methodsFor: 'accessing' stamp: 'lr 11/6/2011 17:29'!
at: anInteger put: anObject
	| node |
	node := self entryAt: anInteger.
	node isNil
		ifTrue: [ ^ self indexOutOfBounds: anInteger ].
	node object: anObject.
	^ anObject! !

!CTLinkedList methodsFor: 'initialization' stamp: 'lr 11/6/2011 17:35'!
initialize
	super initialize.
	root := self nodeClass new.
	root next: root; prev: root.
	size := 0! !

!CTLinkedList methodsFor: 'accessing' stamp: 'lr 11/6/2011 17:29'!
iterator
	self shouldBeImplemented! !

!CTLinkedList methodsFor: 'private' stamp: 'lr 11/6/2011 17:36'!
nodeAdd: aNode before: anotherNode
	aNode prev: anotherNode prev.
	aNode next: anotherNode.
	aNode prev next: aNode.
	aNode next prev: aNode.
	size := size + 1! !

!CTLinkedList methodsFor: 'private' stamp: 'lr 11/6/2011 17:36'!
nodeAt: anInteger
	| node |
	(anInteger between: 1 and: size)
		ifFalse: [ ^ nil ].
	node := root.
	anInteger < (size // 2)
		ifTrue: [ 1 to: anInteger do: [ :index | node := node next ] ]
		ifFalse: [ 1 to: size - anInteger + 1 do: [ :index | node := node prev ] ].
	^ node! !

!CTLinkedList methodsFor: 'private' stamp: 'lr 11/6/2011 17:07'!
nodeClass
	^ CTLinkedListNode! !

!CTLinkedList methodsFor: 'private' stamp: 'lr 11/6/2011 17:22'!
nodeOn: anObject
	^ self nodeClass on: anObject! !

!CTLinkedList methodsFor: 'private' stamp: 'lr 11/6/2011 17:19'!
nodeRemove: aNode
	aNode before after: aNode after.
	aNode after before: aNode before.
	size := size - 1! !

!CTLinkedList methodsFor: 'removing' stamp: 'lr 11/6/2011 17:33'!
removeAt: anInteger ifAbsent: aBlock
	| node |
	(node := self nodeAt: anInteger) isNil
		ifTrue: [ ^ self indexOutOfBounds: anInteger ].
	self removeNode: node.
	^ node object! !

!CTLinkedList methodsFor: 'removing' stamp: 'lr 11/6/2011 17:32'!
removeFirst
	| node |
	self isEmpty
		ifTrue: [ ^ self noSuchElement ].
	self nodeRemove: (node := root after).
	^ node element! !

!CTLinkedList methodsFor: 'removing' stamp: 'lr 11/6/2011 17:26'!
removeLast
	| node |
	self isEmpty
		ifTrue: [ ^ self noSuchElement ].
	self nodeRemove: (node := root before).
	^ node element! !

!CTLinkedList methodsFor: 'accessing' stamp: 'lr 11/6/2011 17:29'!
size
	^ size! !

!CTList methodsFor: 'adding' stamp: 'lr 11/6/2011 09:34'!
add: anObject
	"Appends anObject to the receiver."
	
	self addLast: anObject! !

!CTList methodsFor: 'adding' stamp: 'lr 11/6/2011 09:34'!
add: anObject at: anInteger
	"Adds anObject at the position anInteger."
	
	self subclassResponsibility! !

!CTList methodsFor: 'adding' stamp: 'lr 11/6/2011 09:33'!
addFirst: anObject
	"Adds anObject at the beginning of the receiver."
	
	self add: anObject at: 1! !

!CTList methodsFor: 'adding' stamp: 'lr 11/6/2011 09:34'!
addLast: anObject
	"Adds anObject at the end of the receiver."
	
	self add: anObject at: self size + 1! !

!CTList methodsFor: 'accessing' stamp: 'lr 8/7/2011 16:14'!
at: anInteger
	"Returns the element at index anInteger, or throws an exception."
	
	^ self at: anInteger ifAbsent: [ self indexOutOfBounds: anInteger ]! !

!CTList methodsFor: 'accessing' stamp: 'lr 8/7/2011 15:56'!
at: anInteger ifAbsent: aBlock
	"Returns the element at anIndex, otherwise answer the result of evaluating aBlock."
	
	self subclassResponsibility! !

!CTList methodsFor: 'accessing' stamp: 'lr 8/7/2011 15:57'!
at: anInteger put: anObject
	"Replaces the element at anInteger with anObject."
	
	self subclassResponsibility! !

!CTList methodsFor: 'accessing' stamp: 'lr 8/7/2011 19:55'!
first
	"Answers the first element of the collection."
	
	^ self at: 1 ifAbsent: [ self noSuchElement ]! !

!CTList methodsFor: 'accessing' stamp: 'lr 8/7/2011 19:55'!
last
	"Answers the last element of the collection."
	
	^ self at: self size ifAbsent: [ self noSuchElement ]! !

!CTList methodsFor: 'removing' stamp: 'lr 11/6/2011 09:37'!
removeAt: anInteger
	"Removes the element at index anInteger."
	
	self removeAt: anInteger ifAbsent: [ self indexOutOfBounds: anInteger ]! !

!CTList methodsFor: 'removing' stamp: 'lr 11/6/2011 09:36'!
removeAt: anInteger ifAbsent: aBlock
	"Removes the element at index anInteger."
	
	self subclassResponsibility! !

!CTList methodsFor: 'removing' stamp: 'lr 11/6/2011 09:37'!
removeFirst
	"Removes the first element of the receiver."
	
	^ self removeAt: 1! !

!CTList methodsFor: 'removing' stamp: 'lr 11/6/2011 09:37'!
removeLast
	"Removes the last element of the receiver."
	
	^ self removeAt: self size! !

CTCollection subclass: #CTSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Abstract'!

CTSet subclass: #CTHashSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Sets'!

CTHashSet subclass: #CTLinkedHashSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Sets'!

CTSet subclass: #CTTreeSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Sets'!

!CTTreeSet class methodsFor: 'as yet unclassified' stamp: 'lr 7/13/2011 09:17'!
foo
	^ Fooo! !

!CTTreeSet methodsFor: 'as yet unclassified' stamp: 'lr 7/13/2011 09:29'!
foo
	^ Fooo + tally + Object! !

Object subclass: #CTHashEntry
	instanceVariableNames: 'key keyHash value next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Maps'!

!CTHashEntry class methodsFor: 'instance creation' stamp: 'lr 6/7/2011 20:30'!
key: aKey hash: aHash value: aValue next: anEntry
	^ self basicNew initializeKey: aKey hash: aHash value: aValue next: anEntry! !

!CTHashEntry methodsFor: 'initialization' stamp: 'lr 7/13/2011 12:06'!
initializeKey: aKey hash: anInteger value: aValue next: anEntry
	key := aKey.
	keyHash := anInteger.
	value := aValue.
	next := anEntry! !

!CTHashEntry methodsFor: 'accessing' stamp: 'lr 6/7/2011 20:32'!
key
	^ key! !

!CTHashEntry methodsFor: 'accessing' stamp: 'lr 6/7/2011 20:31'!
keyHash
	^ keyHash! !

!CTHashEntry methodsFor: 'accessing' stamp: 'lr 7/13/2011 12:06'!
next
	^ next! !

!CTHashEntry methodsFor: 'modifying' stamp: 'lr 6/7/2011 20:32'!
replaceValue: aValue
	| oldValue |
	oldValue := value.
	value := aValue.
	^ oldValue! !

!CTHashEntry methodsFor: 'accessing' stamp: 'lr 6/7/2011 20:32'!
value
	^ value! !

CTHashEntry subclass: #CTLinkedHashEntry
	instanceVariableNames: 'before after'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Maps'!

Object subclass: #CTIterator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Iterators'!

CTIterator subclass: #CTArrayListIterator
	instanceVariableNames: 'sequence position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Lists'!

!CTArrayListIterator class methodsFor: 'instance creation' stamp: 'lr 8/3/2011 19:53'!
on: aSequenceableCollection
	^ self basicNew initializeOn: aSequenceableCollection! !

!CTArrayListIterator methodsFor: 'testing' stamp: 'lr 8/3/2011 19:53'!
hasNext
	^ position < sequence size! !

!CTArrayListIterator methodsFor: 'initialization' stamp: 'lr 8/3/2011 19:53'!
initializeOn: aSequencableCollection
	self initialize.
	sequence := aSequencableCollection.
	position := 0! !

!CTArrayListIterator methodsFor: 'accessing' stamp: 'lr 8/3/2011 19:52'!
next
	self hasNext
		ifFalse: [ ^ self noSuchElementError ].
	^ sequence at: (position := position + 1)! !

CTIterator subclass: #CTChainingIterator
	instanceVariableNames: 'iterators current'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Iterators'!

!CTChainingIterator class methodsFor: 'instance creation' stamp: 'lr 8/3/2011 19:55'!
on: anIterator
	^ self basicNew initializeOn: anIterator! !

!CTChainingIterator methodsFor: 'testing' stamp: 'lr 8/3/2011 19:17'!
hasNext
	| hasNext |
	[ (hasNext := current hasNext) not and: [ iterators hasNext ] ]
		whileTrue: [ current := iterators next ].
	^ hasNext! !

!CTChainingIterator methodsFor: 'initialization' stamp: 'lr 8/3/2011 19:56'!
initializeOn: anIterator
	self initialize.
	iterators := anIterator.
	current := CTEmptyIterator new! !

!CTChainingIterator methodsFor: 'accessing' stamp: 'lr 8/3/2011 19:47'!
next
	self hasNext
		ifFalse: [ ^ self noSuchElementError ].
	^ current next! !

CTIterator subclass: #CTDelegateIterator
	instanceVariableNames: 'iterator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Iterators'!

CTDelegateIterator subclass: #CTCyclingIterator
	instanceVariableNames: 'current'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Iterators'!

!CTCyclingIterator commentStamp: 'lr 8/3/2011 18:08' prior: 0!
An iterator that cycles infinitely over its elements.!

!CTCyclingIterator methodsFor: 'testing' stamp: 'lr 8/3/2011 18:17'!
hasNext
	current hasNext
		ifFalse: [ current := iterator copy ].
	^ current hasNext! !

!CTCyclingIterator methodsFor: 'initialization' stamp: 'lr 8/3/2011 19:57'!
initializeOn: anIterator
	super initializeOn: anIterator.
	current := anIterator copy! !

!CTCyclingIterator methodsFor: 'accessing' stamp: 'lr 8/3/2011 19:48'!
next
	self hasNext
		ifFalse: [ ^ self noSuchElementError ].
	^ current next! !

!CTDelegateIterator class methodsFor: 'instance creation' stamp: 'lr 8/3/2011 19:21'!
on: anInterator
	^ self basicNew initializeOn: anInterator! !

!CTDelegateIterator methodsFor: 'testing' stamp: 'lr 8/3/2011 18:22'!
hasNext
	^ iterator hasNext! !

!CTDelegateIterator methodsFor: 'initialization' stamp: 'lr 8/3/2011 19:21'!
initializeOn: anIterator
	self initialize.
	iterator := anIterator! !

!CTDelegateIterator methodsFor: 'accessing' stamp: 'lr 8/3/2011 18:22'!
next
	^ iterator next! !

CTDelegateIterator subclass: #CTLimitingIterator
	instanceVariableNames: 'limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Iterators'!

!CTLimitingIterator methodsFor: 'testing' stamp: 'lr 8/3/2011 19:19'!
hasNext
	^ 0 < limit and: [ super hasNext ]! !

!CTLimitingIterator methodsFor: 'accessing' stamp: 'lr 8/3/2011 19:19'!
next
	self hasNext
		ifFalse: [ ^ self noSuchElementError ].
	limit := limit - 1.
	^ super next! !

!CTLimitingIterator methodsFor: 'initialization' stamp: 'lr 8/3/2011 18:27'!
setLimit: anInteger
	limit := anInteger! !

CTDelegateIterator subclass: #CTMutatingIterator
	instanceVariableNames: 'block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Iterators'!

!CTMutatingIterator methodsFor: 'accessing' stamp: 'lr 8/3/2011 18:21'!
next
	^ block value: iterator next! !

CTDelegateIterator subclass: #CTPeekingIterator
	instanceVariableNames: 'element'
	classVariableNames: 'Sentinel'
	poolDictionaries: ''
	category: 'Container-Iterators'!

CTPeekingIterator subclass: #CTFilterIterator
	instanceVariableNames: 'predicate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Iterators'!

!CTFilterIterator class methodsFor: 'as yet unclassified' stamp: 'lr 8/3/2011 07:30'!
initialize
	Sentinel := Object new! !

!CTFilterIterator methodsFor: 'testing' stamp: 'lr 8/3/2011 19:44'!
hasNext
	| next |
	[ element == Sentinel and: [ iterator hasNext ] ] whileTrue: [
		next := iterator next.
		(predicate value: next)
			ifTrue: [ element := next ] ].
	^ element ~~ Sentinel! !

!CTFilterIterator methodsFor: 'initialization' stamp: 'lr 8/3/2011 19:41'!
setPredicate: aValuable
	predicate := aValuable! !

!CTPeekingIterator class methodsFor: 'initialization' stamp: 'lr 8/3/2011 19:02'!
initialize
	Sentinel := Object new! !

!CTPeekingIterator methodsFor: 'testing' stamp: 'lr 8/3/2011 19:38'!
hasNext
	(element == Sentinel and: [ iterator hasNext ])
		ifTrue: [ element := iterator next ].
	^ element ~~ Sentinel! !

!CTPeekingIterator methodsFor: 'initialization' stamp: 'lr 8/3/2011 19:02'!
initialize
	super initialize.
	element := Sentinel! !

!CTPeekingIterator methodsFor: 'accessing' stamp: 'lr 8/3/2011 19:40'!
next
	| next |
	self hasNext
		ifFalse: [ ^ self noSuchElement ].
	next := element.
	element := Sentinel.
	^ next! !

!CTPeekingIterator methodsFor: 'accessing' stamp: 'lr 8/3/2011 19:40'!
peek
	"Answer the next element without actually consuming it, or raise an CTNoSuchElementError."
	
	self hasNext
		ifFalse: [ ^ self noSuchElement ].
	^ element! !

CTIterator subclass: #CTEmptyIterator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Iterators'!

!CTEmptyIterator methodsFor: 'testing' stamp: 'lr 8/3/2011 19:01'!
hasNext
	^ false! !

!CTEmptyIterator methodsFor: 'accessing' stamp: 'lr 8/3/2011 19:14'!
next
	^ self noSuchElementError! !

!CTIterator methodsFor: 'enumerating' stamp: 'lr 8/3/2011 07:18'!
do: aBlock
	[ self hasNext ]
		whileTrue: [ aBlock value: self next ]! !

!CTIterator methodsFor: 'testing' stamp: 'lr 8/3/2011 19:01'!
hasNext
	"Answer whether there is a next element in the iterator."
	
	self subclassResponsibility! !

!CTIterator methodsFor: 'accessing' stamp: 'lr 8/3/2011 19:16'!
next
	"Answer the next element in the iterator, or raise CTNoSuchElementError if no such element exists."
	
	self subclassResponsibility! !

!CTIterator methodsFor: 'private' stamp: 'lr 8/3/2011 19:14'!
noSuchElementError
	^ CTNoSuchElementError signal! !

Object subclass: #CTLinkedListNode
	instanceVariableNames: 'element before after object next prev'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Lists'!

!CTLinkedListNode class methodsFor: 'instance creation' stamp: 'lr 11/6/2011 17:27'!
on: anObject
	^ self basicNew object: anObject! !

!CTLinkedListNode methodsFor: 'accessing' stamp: 'lr 11/6/2011 17:26'!
next
	^ next! !

!CTLinkedListNode methodsFor: 'accessing' stamp: 'lr 11/6/2011 17:26'!
next: aNode
	next := aNode! !

!CTLinkedListNode methodsFor: 'accessing' stamp: 'lr 11/6/2011 17:26'!
object
	^ object! !

!CTLinkedListNode methodsFor: 'accessing' stamp: 'lr 11/6/2011 17:26'!
object: anObject
	object := anObject! !

!CTLinkedListNode methodsFor: 'accessing' stamp: 'lr 11/6/2011 17:27'!
prev
	^ prev! !

!CTLinkedListNode methodsFor: 'accessing' stamp: 'lr 11/6/2011 17:27'!
prev: aNode
	prev := aNode! !

Object subclass: #CTMap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Abstract'!

CTMap subclass: #CTHashMap
	instanceVariableNames: 'size modifications entries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Maps'!

!CTHashMap class methodsFor: 'instance creation' stamp: 'lr 7/13/2011 12:18'!
new
	^ self new: 16! !

!CTHashMap class methodsFor: 'instance creation' stamp: 'lr 7/13/2011 12:18'!
new: anInteger
	^ self basicNew
		initialize: anInteger;
		yourself! !

!CTHashMap methodsFor: 'querying' stamp: 'lr 6/7/2011 20:18'!
at: aKey ifAbsent: aBlock
	| hash entry |
	hash := self hashFor: aKey.
	entry := entries at: (self indexFor: hash).
	[ entry isNil ] whileFalse: [
		(entry hash = hash and: [ self compare: aKey with: entry key ])
			ifTrue: [ ^ entry value ].
		entry := entry next ].
	^ aBlock value! !

!CTHashMap methodsFor: 'modifying' stamp: 'lr 7/13/2011 12:22'!
at: aKey put: aValue
	| hash index entry |
	hash := self hashFor: aKey.
	index := self indexFor: hash.
	entry := entries at: index.
	[ entry isNil ] whileFalse: [
		(entry hash = hash and: [ self compare: aKey with: entry key ]) ifTrue: [
			entry setValue: aValue.
			^ aValue ] ].
	entries at: index put: (CTHashEntry key: aKey value: aValue hash: hash next: (entries at: index)).
	size := size + 1.
	^ aValue! !

!CTHashMap methodsFor: 'private' stamp: 'lr 11/5/2011 19:15'!
capacityFor: anInteger
	| capacity |
	capacity := 1.
	[ capacity < anInteger ]
		whileTrue: [ capacity := 2 * capacity ].
	^ capacity! !

!CTHashMap methodsFor: 'private' stamp: 'lr 6/7/2011 20:18'!
compare: anObject with: anotherObject
	^ anObject = anotherObject! !

!CTHashMap methodsFor: 'private' stamp: 'lr 11/5/2011 19:15'!
hashFor: aKey
	^ aKey hash! !

!CTHashMap methodsFor: 'private' stamp: 'lr 11/5/2011 19:15'!
indexFor: anInteger
	^ anInteger bitAnd: entries size - 1! !

!CTHashMap methodsFor: 'initialization' stamp: 'lr 7/13/2011 12:23'!
initialize: anInteger
	size := modifications := 0.
	entries := Array new: (self capacityFor: anInteger)! !

!CTHashMap methodsFor: 'private' stamp: 'lr 7/13/2011 12:11'!
loadFactor
	^ 0.75! !

!CTHashMap methodsFor: 'accessing' stamp: 'lr 6/7/2011 20:23'!
size
	^ size! !

CTHashMap subclass: #CTLinkedHashMap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Maps'!

!CTMap methodsFor: 'querying' stamp: 'lr 6/7/2011 20:19'!
at: aKey
	^ self at: aKey ifAbsent: [ self keyNotFound ]! !

!CTMap methodsFor: 'querying' stamp: 'lr 6/7/2011 20:19'!
at: aKey ifAbsent: aBlock
	self subclassResponsibility! !

!CTMap methodsFor: 'querying' stamp: 'lr 6/7/2011 20:21'!
at: aKey ifPresent: aBlock
	^ aBlock value: (self at: aKey ifAbsent: [ ^ nil ])! !

!CTMap methodsFor: 'modifying' stamp: 'lr 6/7/2011 20:22'!
at: aKey put: aValue
	self subclassResponsibility! !

!CTMap methodsFor: 'modifying' stamp: 'lr 7/13/2011 12:24'!
clear
	self subclassResponsibility! !

!CTMap methodsFor: 'accessing' stamp: 'lr 6/7/2011 20:22'!
entries
	^ self subclassResponsibility! !

!CTMap methodsFor: 'testing' stamp: 'lr 6/7/2011 20:22'!
includesKey: aKey
	^ self keys includes: aKey! !

!CTMap methodsFor: 'testing' stamp: 'lr 6/7/2011 20:22'!
includesValue: aValue
	^ self values includes: aValue! !

!CTMap methodsFor: 'testing' stamp: 'lr 6/7/2011 20:23'!
isEmpty
	^ self size = 0! !

!CTMap methodsFor: 'accessing' stamp: 'lr 6/7/2011 20:22'!
keys
	^ self subclassResponsibility! !

!CTMap methodsFor: 'accessing' stamp: 'lr 6/7/2011 20:23'!
size
	self subclassResponsibility! !

!CTMap methodsFor: 'accessing' stamp: 'lr 6/7/2011 20:23'!
values
	^ self subclassResponsibility! !

CTMap subclass: #CTTreeMap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Maps'!

Error subclass: #CTElementNotFoundError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Exceptions'!

Error subclass: #CTIndexOutOfBoundsError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Exceptions'!

Error subclass: #CTNoSuchElementError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Container-Exceptions'!
CTPeekingIterator initialize!
CTFilterIterator initialize!
